{$A+,B-,D+,E+,F-,I+,L+,N+,O-,R-,S+,V+}
{$M 16384,0,655360}

{***************************************************************}
{                                                               }
{               TURBO GRAPHIC TOOLS  version 5.00               }
{                                                               }
{                         GRAPHLIB UNIT                         }
{                                                               }
{   Graphics Library Supporting CGA (640x200), EGA (640x350),   }
{   VGA (640x480),  Hercules (720x348) Graphics Adapters, and   }
{   Hewlett-Packard Graphics Language plotters (10000x7200)     }
{                                                               }
{                   by  William F. Polik                        }
{                                                               }
{       (C) Copyright  William F. Polik, 1986, 1987, 1989,      }
{                      1990, 1991                               }
{                                                               }
{  Distributed By:  William F. Polik                            }
{                   Department of Chemistry                     }
{                   Hope College                                }
{                   Holland, MI 94720                           }
{                   tel:  (616) 394-7639                        }
{                                                               }
{***************************************************************}


{NOTE: Plotter is a conditional compilation symbol.  If it is defined,
 code is generated to support a Hewlett-Packard plotter using HPGL.}

{$DEFINE Plotter}

{NOTE: NoHercules is a conditional compilation symbol.  If it is defined,
 no loading of a character font from disk is performed; hence, no
 characters can be drawn to the graphics screen.  It is recommended
 that programs using this symbol trap the presence of the Hercules
 adapter and terminate.}

{$DEFINE NoHercules}

{NOTE: DirectEGAVGAdp is a conditional compilation symbol.  If it is
 defined, code is generated to support direct access of video memory on
 EGA256 and VGA adapters with color monitors.}

{$DEFINE DirectEGAVGAdp}

{NOTE: WordPerfBugFix is a conditional compilation symbol.  If it is
 defined, code is generated to work around a WordPerfect label positioning
 bug (cursor position is not updated after plotting vertical labels).  This
 bug was claimed to be fixed (but not tested by WFP) in the 12/30/90 release.
 However, the 12/30/90 release of WordPerfect does not support symbols.}

{$DEFINE WordPerfBugFix}
{$DEFINE NoPrinter}

unit GraphLib;


interface


uses
  Crt,                    {for Sound,Delay,NoSound,Kepressed,ReadKey}
                          {procedures}
{$IFDEF NoPrinter}
  Dos;                    {for Registers type and Intr,MsDOS procedures}
{$ENDIF}
{$IFNDEF NoPrinter}
  Dos,
  Printer;                {for Lst device in Hardcopy}
{$ENDIF}

const
  Black        = 0;       {color constants}
  Blue         = 1;
  Green        = 2;
  Cyan         = 3;
  Red          = 4;
  Magenta      = 5;
  Brown        = 6;
  LightGray    = 7;
  DarkGray     = 8;
  LightBlue    = 9;
  LightGreen   = 10;
  LightCyan    = 11;
  LightRed     = 12;
  LightMagenta = 13;
  Yellow       = 14;
  White        = 15;
  XORcolor     = 255;     {fixed; do not change value}
  Horizontal   = 0;       {optional constants for SetCharDirection}
  VerticalUp   = 1;
  UpSideDown   = 2;
  VerticalDown = 3;



type
  GraphicsString = String[80];
  AdapterType    = (NoAdapter, PCjrA, MonoA, CGA, EGA64, EGA128,
                    EGA192, EGA256, VGA, Hercules);
  DisplayType    = (NoDisplay, PCjr, Mono, Color, Enhanced, AnalogMono,
                    AnalogColor);
{$IFDEF Plotter}
  HPString       = String[90];
{$ENDIF}

{NOTE: Global variables are provided for READ ONLY purposes, i.e., so
 that a program can find out how many graphic pages are available or
 scale properly as outlined in the Maintaining Transportability section
 of the manual.  The values of the global variables are maintained by
 the PlotLib unit; a program must not alter their values.}

var
  GraphicsOn       : Boolean;          {Flag true if in graphics mode}
  DisplayPage      : Integer;          {Current display page}
  DrawPage         : Integer;          {Current draw page}
  DrawColor        : Integer;          {Current drawing color}
  MaxPage          : Integer;          {Maximum graphics pages in system}
  CharHeight       : Integer;          {Pixel height of text, 8 or 14}
  CharWidth        : Integer;          {Pixel width of text, 8 or 9}
  CharDirection    : Integer;          {Direction code for text}
  AspectFactor     : Real;             {Aspect factor of monitor}
  Adapter          : AdapterType;      {System graphics adapter}
  Display          : DisplayType;      {System display monitor}
  LLXABS,LLYABS,
  URXABS,URYABS    : Integer;          {Screen corners in pixels}
  LLXWIN,LLYWIN,
  URXWIN,URYWIN    : Integer;          {Current window corners in pixels}
  LLXSCR,LLYSCR,
  URXSCR,URYSCR    : Real;             {Screen corners in screen coords}
  LLXUSER,LLYUSER,
  URXUSER,URYUSER  : Real;             {Window corners in user coords}
{$IFDEF Plotter}
  HPPlotFileOpen   : Boolean;          {True if HP plot file is open}
  HPPlotterOn      : Boolean;          {True if HP is plotting graphics}
  HPLLXABS,HPLLYABS,
  HPURXABS,HPURYABS: Integer;          {HP corners in pixels}
  HPLLXWIN,HPLLYWIN,
  HPURXWIN,HPURYWIN: Integer;          {HP window corners in pixels}
{$ENDIF}


procedure ExitGraphics;
procedure GraphicsError(Message: GraphicsString);
{$IFDEF Plotter}
procedure WritePlotter(CommandString: HPString);
{$ENDIF}
procedure SetClippingOn;
procedure SetClippingOff;
procedure SetClearOn;
procedure SetClearOff;
procedure SetHardCopyCancelOn;
procedure SetHardCopyCancelOff;
procedure SetHardwareOverideOn(OverideAdapter: AdapterType;
                               OverideDisplay: DisplayType);
procedure SetHardwareOverideOff;
procedure SetDrawPage(NewPage: Integer);
procedure SetDisplayPage(NewPage: Integer);
procedure SetColor(NewColor: Integer);
procedure SetTextBlockColor(NewColor: Integer);
procedure SetTextDirection(NewDirection: Integer);
{$IFDEF Plotter}
procedure OpenPlotter(FileName: HPString);
procedure ClosePlotter;
procedure SetPlotterOn;
procedure SetPlotterOff;
{$ENDIF}
procedure EnterGraphics;
procedure DefineSymbol(i: Integer; a,b,c,d,e,f,g: Byte);
procedure DefineWindow(LLX,LLY,URX,URY: Real);
procedure DefineUserCoordinates(LLX,LLY,URX,URY: Real);
procedure InitGraphics;
procedure MoveUser(Xuser,Yuser: Real);
procedure MoveScreen(Xscreen,Yscreen: Real);
procedure DrawLineUser(Xuser,Yuser: Real);
procedure DrawLineScreen(Xscreen,Yscreen: Real);
procedure MoveRelUser(XrelUser,YrelUser: Real);
procedure MoveRelScreen(XrelScreen,YrelScreen: Real);
procedure DrawLineRelUser(XrelUser,YrelUser: Real);
procedure DrawLineRelScreen(XrelScreen,YrelScreen: Real);
procedure DrawPoint;
procedure DrawSymbol(i: Integer);
procedure DrawCircleScreen(rscreen:Real);
procedure DrawCircleUser(ruser:Real);
procedure ReplaceFill;
procedure BorderFill(BorderColor: Integer);
procedure DrawText(Message: GraphicsString);
procedure GetText(var Message: GraphicsString; MaxLength: Integer);
{$IFNDEF NoPrinter}
procedure HardCopy;
{$ENDIF}

implementation


const
  MaxSymbol    = 9;       {0..any positive integer allowable}
  MaxHercPage  = 1;       {0 or 1 allowable; 0 uses $B0000 - $B7FFF}
                          {(1 page); 1 uses $B0000 - $BFFFF (2 pages)}
  LLXScreen    = 0;       {screen coordinate system defined here;}
  LLYScreen    = 0;       {may be changed by the user}
  URXScreen    = 1000;
  URYScreen    = 1000;
{$IFDEF Plotter}
  LLXHP        = 250;     {HP absolute coordinate system defined here;}
  LLYHP        = 279;     {may be varied to change size of plot}
  URXHP        = 10250;
  URYHP        = 7479;
{$ENDIF}

type
  SymbolMask     = Array[0..6] of Byte;
  FillType       = (Replace, Border);
{$IFDEF Plotter}
  HPIString      = String[6];
  HPRString      = String[8];
{$ENDIF}

var
  SaveExit         : Pointer;          {Exit procedure chain pointer}
  Regs             : Registers;        {8088 internal registers}
  TextMode         : Integer;          {Text mode of system}
  GraphicsMode     : Integer;          {Best graphics mode of system}
  HercSegment      : Word;             {Segment of Hercules DrawPage}
  ClearOn          : Boolean;          {Flag true if SetClearOn}
  ClippingOn       : Boolean;          {Flag true if SetClippingOn}
  HardCopyCancelOn : Boolean;          {Flag true if HardCopyCancelOn}
  HardwareOvrdOn   : Boolean;          {Flag true if HardwareOvrdOn}
  FillStyle        : FillType;         {Flag indicating fill type}
  FillColor        : Integer;          {Color for filling routine}
  Filli            : Integer;          {Index for filling routine}
  CharBlockColor   : Integer;          {Current text block color}
  CharROMSegment   : Word;             {Segment of text character fonts}
  CharROMOffset    : Word;             {Offset of text character fonts}
  Symbol           : Array[0..MaxSymbol] of SymbolMask; {Symbol patterns}
  OvrdAdapter      : AdapterType;      {Overide graphics adapter}
  OvrdDisplay      : DisplayType;      {Overide display monitor}
  XabsGbl,YabsGbl  : Integer;          {Current graphics cursor location}
  XminABS,YminABS,
  XmaxABS,YmaxABS  : Integer;          {Screen limits in pixels}
  XminWIN,YminWIN,
  XmaxWIN,YmaxWIN  : Integer;          {Window limits in pixels}
{$IFDEF Plotter}
  HPPenUp          : Boolean;          {True if HP pen is up}
  HPSymbol         : Array[0..MaxSymbol] of Char; {HP symbol characters}
  HPPlotFile       : Text;             {HP plotter command file}
{$ENDIF}


{$I GRAPHLIB.1}  {GRAPHLIB.PAS Environmental Procedures}


{$I GRAPHLIB.2}  {GRAPHLIB.PAS Plotting Procedures}


begin {of initialization code}
SaveExit := ExitProc;
ExitProc := @GraphicsExit;
SetHardwareOverideOff;
GraphicsOn := False;
{$IFDEF Plotter}
HPPlotFileOpen := False;
{$ENDIF}
end. {of initialization code}
